package falkor.ui;

import java.awt.AWTEvent;
import java.awt.Canvas;
import java.awt.Dimension;
import java.awt.event.ComponentEvent;
import java.awt.event.FocusEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseWheelEvent;
import java.awt.event.WindowEvent;
import java.util.concurrent.LinkedBlockingQueue;

import falkore.core.GameState;
import falkore.core.GameState.State;


public class InputThread extends Thread {
   private LinkedBlockingQueue<AWTEvent> eventQueue;
   private GameState gameState;
   private Canvas gameCanvas;

   public InputThread(
         LinkedBlockingQueue<AWTEvent> messageQueue, 
         GameState gameState, 
         Canvas gameCanvas) {
      this.eventQueue = messageQueue;
      this.gameState = gameState;
      this.gameCanvas = gameCanvas;
      this.setName("InputThread");
   }

   public void run() {
      do {	
         try {
            AWTEvent ev;
            if((
                  !eventQueue.isEmpty() && 
                  ( ev = eventQueue.take() ) != null )) {
               processEvent(ev);
            }
         } catch (InterruptedException ie) {
            System.out.println("synch conflict: dropping input tick: " + ie);
         }
      } while( gameState.getState() != GameState.State.STOP );
   }

   private void processEvent(AWTEvent ev) {
      if( ev instanceof WindowEvent ) {
         processEvent( (WindowEvent) ev );
      } else if( ev instanceof MouseWheelEvent ) {
         processEvent( (MouseWheelEvent) ev );
      } else if( ev instanceof KeyEvent ) {
         processEvent( (KeyEvent) ev );
      } else if( ev instanceof MouseEvent ) {
         processEvent( (MouseEvent) ev );
      } else if( ev instanceof ComponentEvent ) {
         processEvent( (ComponentEvent) ev );
      } else if( ev instanceof FocusEvent ) {
         processEvent( (FocusEvent) ev );
      }
   }

   private void processEvent( WindowEvent wev ) {
      switch( wev.getID() ) {
      case WindowEvent.WINDOW_CLOSING:
         gameState.updateKeypressMap(KeyEvent.VK_ESCAPE, true);
         break;
      }
   }

   private void processEvent( MouseWheelEvent mwev ) {		
      gameState.updateMouseWheelDelta(mwev.getWheelRotation());
   }

   private void processEvent( KeyEvent kev ) {
      if( kev.getID() == KeyEvent.KEY_PRESSED ) {
         gameState.updateKeypressMap(kev.getKeyCode(), true);
      } else if( kev.getID() == KeyEvent.KEY_RELEASED ) {
         gameState.updateKeypressMap(kev.getKeyCode(), false);
      }
   }

   private void processEvent( MouseEvent mev ) {
      Dimension canvasDim = gameCanvas.getSize();
      Dimension gameDim = gameState.getGameArea();
      double scaleX = (double)gameDim.width/canvasDim.width;
      double scaleY = (double)gameDim.height/canvasDim.height;

      gameState.updateMousePosition(
            (int)(mev.getX()*scaleX), 
            (int)(mev.getY()*scaleY));

      if( mev.getID() == MouseEvent.MOUSE_PRESSED ) {
         gameState.updateMouseMap(mev.getButton(), true);
      } else if( mev.getID() == MouseEvent.MOUSE_RELEASED ) {
         gameState.updateMouseMap(mev.getButton(), false);
      }
   }

   private void processEvent( ComponentEvent ce ) {
      switch( ce.getID() ) {
      case ComponentEvent.COMPONENT_RESIZED:
      case ComponentEvent.COMPONENT_HIDDEN:
         gameState.pause();
         break;
      }
   }

   private void processEvent( FocusEvent fe ) {
      if(fe.getID() == FocusEvent.FOCUS_LOST ) {
         gameState.pause();
      }
   }
}
