package falkor.ui;

import java.awt.AWTEvent;
import java.awt.Canvas;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.util.concurrent.LinkedBlockingQueue;

import javax.swing.JFrame;

@SuppressWarnings("serial")
public class MainWindow extends JFrame implements 
KeyListener, 
MouseListener, 
MouseMotionListener, 
MouseWheelListener, 
WindowListener, 
ComponentListener,
FocusListener {
   public static int GAME_WINDOW_WIDTH = 512;
   public static int GAME_WINDOW_HEIGHT = 316;
   private LinkedBlockingQueue<AWTEvent> eventQueue;

   public MainWindow( 
         Canvas drawCanvas, 
         LinkedBlockingQueue<AWTEvent> eventQueue ) {
      this.eventQueue = eventQueue;

      addWindowListener(this);
      addComponentListener(this);
      drawCanvas.addFocusListener(this);
      drawCanvas.addKeyListener(this);
      drawCanvas.addMouseListener(this);
      drawCanvas.addMouseMotionListener(this);
      drawCanvas.addMouseWheelListener(this);

      setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);

      setTitle("falkor");
      drawCanvas.setSize(GAME_WINDOW_WIDTH, GAME_WINDOW_HEIGHT);
      add(drawCanvas);
      pack();
   }

   private void postMessage(AWTEvent msg) {
      /* TODO: cache these messages internally in a stack */
      try {
         eventQueue.put(msg);
      } catch (InterruptedException e) {
         e.printStackTrace();
      }
   }

   @Override
   public void mouseDragged(MouseEvent e) {
      postMessage(e);
   }

   @Override
   public void mouseMoved(MouseEvent e) {
      postMessage(e);
   }

   @Override
   public void mouseClicked(MouseEvent e) {
      postMessage(e);
   }

   @Override
   public void mousePressed(MouseEvent e) {
      postMessage(e);
   }

   @Override
   public void mouseReleased(MouseEvent e) {
      postMessage(e);
   }

   @Override
   public void mouseEntered(MouseEvent e) {
      postMessage(e);
   }

   @Override
   public void mouseExited(MouseEvent e) {
      postMessage(e);
   }

   @Override
   public void keyTyped(KeyEvent e) {
      postMessage(e);
   }

   @Override
   public void keyPressed(KeyEvent e) {
      postMessage(e);
   }

   @Override
   public void keyReleased(KeyEvent e) {
      postMessage(e);
   }

   @Override
   public void mouseWheelMoved(MouseWheelEvent e) {
      postMessage(e);
   }

   @Override
   public void windowOpened(WindowEvent e) {
      postMessage(e);
   }

   @Override
   public void windowClosing(WindowEvent e) {
      postMessage(e);
   }

   @Override
   public void windowClosed(WindowEvent e) {
      postMessage(e);
   }

   @Override
   public void windowIconified(WindowEvent e) {
      postMessage(e);
   }

   @Override
   public void windowDeiconified(WindowEvent e) {
      postMessage(e);
   }

   @Override
   public void windowActivated(WindowEvent e) {
      postMessage(e);
   }

   @Override
   public void windowDeactivated(WindowEvent e) {
      postMessage(e);
   }

   @Override
   public void componentResized(ComponentEvent e) {
      postMessage(e);
   }

   @Override
   public void componentMoved(ComponentEvent e) {
      postMessage(e);
   }

   @Override
   public void componentShown(ComponentEvent e) {
      postMessage(e);
   }

   @Override
   public void componentHidden(ComponentEvent e) {
      postMessage(e);
   }

   @Override
   public void focusGained(FocusEvent e) {
      postMessage(e);
   }

   @Override
   public void focusLost(FocusEvent e) {
      postMessage(e);
   }
}
