package falkore.core;

import java.awt.Canvas;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.image.BufferStrategy;


public class RenderThread extends Thread {
   private GameState gameState;
   private Canvas gameCanvas;

   public RenderThread( GameState gameState, Canvas gameCanvas) {
      this.gameState = gameState;
      this.gameCanvas = gameCanvas;
      this.setName("RenderThread");
   }

   public void run() {
      BufferStrategy bufferStrategy = null;

      do {	
         bufferStrategy = gameCanvas.getBufferStrategy();
         renderFrame(bufferStrategy);
         bufferStrategy.show();

      } while( gameState.getState() != GameState.State.STOP );

      if( bufferStrategy != null ) {
         bufferStrategy.dispose();
      }
   }

   private void renderFrame(BufferStrategy bufferStrategy) {
      Dimension canvasDim = gameCanvas.getSize();
      Dimension gameDim = gameState.getGameArea();
      Graphics2D gfx = (Graphics2D) bufferStrategy.getDrawGraphics();

      /* Reset the frame buffer to the background color */
      gfx.setColor(Color.darkGray);
      gfx.fillRect(0, 0, 
            canvasDim.width, 
            canvasDim.height);

      /* Draw current status */
      {
         String statusString = "state: ";

         switch(gameState.getRunState()) {
         case GREET:   statusString += "greet";   break;
         case GAME:    statusString += "game";    break;
         case PAUSE:   statusString += "pause";   break;
         case HISCORE: statusString += "hiscore"; break;
         }

         /* Print debug strings */
         gfx.setColor(Color.green);
         gfx.drawString("state: " + statusString, 15, 15);
      }
 
      /* Scale graphics so that we can use a consistent coordinate system */
      /* FIXME: this doesn't work. need to use affine transform instead */
      
      gfx.scale( 
            (double) canvasDim.width/gameDim.width, 
            (double) canvasDim.height/gameDim.height);

      /* Draw the game objects */
      gameState.render(gfx);
   }
}
